@@ -14,6 +14,30 @@ use std::time::Duration;
 use super::Semphore;
 use crossbeam::queue::SegQueue;
 
+/// Create an unbounded channel. if If you want to limit the number of messages, use bounded channel_buf()
+pub fn channel<T>() -> (Sender<T>, Receiver<T>) {
+    let a = Arc::new(InnerQueue::new());
+    (Sender::new(a.clone()), Receiver::new(a))
+}
+
+/// Create a bounded mpmc channel(More producers, more consumers)
+pub fn channel_buf<T>(buf: usize) -> (Sender<T>, Receiver<T>) {
+    let a = Arc::new(InnerQueue::new_buffer(buf));
+    (Sender::new(a.clone()), Receiver::new(a))
+}
+
+/// Create an unbounded channel. if If you want to limit the number of messages, use bounded channel_buf()
+pub fn unbounded<T>() -> (Sender<T>, Receiver<T>) {
+    channel_buf(usize::MAX)
+}
+
+/// Create a bounded channel
+pub fn bounded<T>(buf: usize) -> (Sender<T>, Receiver<T>) {
+    let a = Arc::new(InnerQueue::new_buffer(buf));
+    (Sender::new(a.clone()), Receiver::new(a))
+}
+
+
 /// /////////////////////////////////////////////////////////////////////////////
 /// InnerQueue
 /// /////////////////////////////////////////////////////////////////////////////
@@ -32,7 +56,6 @@ struct InnerQueue<T> {
 }
 
 impl<T> InnerQueue<T> {
-
     /// default is an Unbounded buffer
     pub fn new() -> InnerQueue<T> {
         InnerQueue::new_buffer(usize::MAX)
@@ -149,6 +172,11 @@ impl<T> InnerQueue<T> {
             n => panic!("bad number of rx_ports left {}", n),
         }
     }
+
+    /// return remain msg len
+    pub fn remain(&self) -> usize {
+        self.queue.len()
+    }
 }
 
 impl<T> Drop for InnerQueue<T> {
@@ -162,6 +190,13 @@ pub struct Receiver<T> {
     inner: Arc<InnerQueue<T>>,
 }
 
+impl<T> Receiver<T> {
+    /// return remain msg len
+    pub fn remain(&self) -> usize {
+        self.inner.remain()
+    }
+}
+
 unsafe impl<T: Send> Send for Receiver<T> {}
 // impl<T> !Sync for Receiver<T> {}
 
@@ -181,33 +216,16 @@ pub struct Sender<T> {
     inner: Arc<InnerQueue<T>>,
 }
 
+impl<T> Sender<T> {
+    /// return remain msg len
+    pub fn remain(&self) -> usize {
+        self.inner.remain()
+    }
+}
+
 unsafe impl<T: Send> Send for Sender<T> {}
 // impl<T> !Sync for Sender<T> {}
 
-/// Create an unbounded channel. if If you want to limit the number of messages, use bounded channel_buf()
-pub fn channel<T>() -> (Sender<T>, Receiver<T>) {
-    let a = Arc::new(InnerQueue::new());
-    (Sender::new(a.clone()), Receiver::new(a))
-}
-
-/// Create a bounded mpmc channel(More producers, more consumers)
-pub fn channel_buf<T>(buf: usize) -> (Sender<T>, Receiver<T>) {
-    let a = Arc::new(InnerQueue::new_buffer(buf));
-    (Sender::new(a.clone()), Receiver::new(a))
-}
-
-/// Create an unbounded channel. if If you want to limit the number of messages, use bounded channel_buf()
-pub fn unbounded<T>() -> (Sender<T>, Receiver<T>) {
-    channel_buf(usize::MAX)
-}
-
-/// Create a bounded channel
-pub fn bounded<T>(buf: usize) -> (Sender<T>, Receiver<T>) {
-    let a = Arc::new(InnerQueue::new_buffer(buf));
-    (Sender::new(a.clone()), Receiver::new(a))
-}
-
-
 
 /// /////////////////////////////////////////////////////////////////////////////
 /// Sender
