@@ -9,9 +9,33 @@ use std::time::{Duration, Instant};
 use crossbeam::queue::SegQueue;
 use crate::coroutine::yield_now;
 use crate::std::sync::Semphore;
-
 use super::{AtomicOption, Blocker};
 
+
+/// Create an unbounded channel. if If you want to limit the number of messages, use bounded channel_buf()
+pub fn channel<T>() -> (Sender<T>, Receiver<T>) {
+    channel_buf(usize::MAX)
+}
+
+/// Create a bounded channel
+pub fn channel_buf<T>(buf: usize) -> (Sender<T>, Receiver<T>) {
+    let a = Arc::new(InnerQueue::new_buffer(buf));
+    (Sender::new(a.clone()), Receiver::new(a))
+}
+
+/// Create an unbounded channel. if If you want to limit the number of messages, use bounded channel_buf()
+pub fn unbounded<T>() -> (Sender<T>, Receiver<T>) {
+    channel_buf(usize::MAX)
+}
+
+/// Create a bounded channel
+pub fn bounded<T>(buf: usize) -> (Sender<T>, Receiver<T>) {
+    let a = Arc::new(InnerQueue::new_buffer(buf));
+    (Sender::new(a.clone()), Receiver::new(a))
+}
+
+
+
 /// /////////////////////////////////////////////////////////////////////////////
 /// InnerQueue
 /// /////////////////////////////////////////////////////////////////////////////
@@ -143,6 +167,11 @@ impl<T> InnerQueue<T> {
         // clear all the data
         while self.queue.pop().is_some() {}
     }
+
+    /// return remain msg len
+    pub fn remain(&self) -> usize {
+        self.queue.len()
+    }
 }
 
 impl<T> Drop for InnerQueue<T> {
@@ -175,6 +204,13 @@ pub struct Sender<T> {
     inner: Arc<InnerQueue<T>>,
 }
 
+impl<T> Sender<T> {
+    /// return remain msg len
+    pub fn remain(&self) -> usize {
+        self.inner.remain()
+    }
+}
+
 unsafe impl<T: Send> Send for Sender<T> {}
 
 // impl<T> !Sync for Sender<T> {}
@@ -182,29 +218,6 @@ impl<T: Send> UnwindSafe for Sender<T> {}
 
 impl<T: Send> RefUnwindSafe for Sender<T> {}
 
-/// Create an unbounded channel. if If you want to limit the number of messages, use bounded channel_buf()
-pub fn channel<T>() -> (Sender<T>, Receiver<T>) {
-    channel_buf(usize::MAX)
-}
-
-/// Create a bounded channel
-pub fn channel_buf<T>(buf: usize) -> (Sender<T>, Receiver<T>) {
-    let a = Arc::new(InnerQueue::new_buffer(buf));
-    (Sender::new(a.clone()), Receiver::new(a))
-}
-
-/// Create an unbounded channel. if If you want to limit the number of messages, use bounded channel_buf()
-pub fn unbounded<T>() -> (Sender<T>, Receiver<T>) {
-    channel_buf(usize::MAX)
-}
-
-/// Create a bounded channel
-pub fn bounded<T>(buf: usize) -> (Sender<T>, Receiver<T>) {
-    let a = Arc::new(InnerQueue::new_buffer(buf));
-    (Sender::new(a.clone()), Receiver::new(a))
-}
-
-
 
 /// /////////////////////////////////////////////////////////////////////////////
 /// Sender
@@ -295,6 +308,11 @@ impl<T> Receiver<T> {
     pub fn try_iter(&self) -> TryIter<T> {
         TryIter { rx: self }
     }
+
+    /// return remain msg len
+    pub fn remain(&self) -> usize {
+        self.inner.remain()
+    }
 }
 
 impl<'a, T> Iterator for Iter<'a, T> {
